# -*- coding: utf8 -*- 
"""A Magazine layout of several Images"""

__author__  = 'nicolas laurance <nlaurance@zindep.com>'
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent

from PIL import Image
try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

try:
  from Products.LinguaPlone.public import *
except ImportError:
  # No multilingual support
  from Products.Archetypes.public import *

from Products.ATContentTypes.content.folder import ATFolder, ATFolderSchema
from Products.ATContentTypes.content.schemata import finalizeATCTSchema

from Products.ATContentTypes.content.schemata import ATContentTypeSchema
from Products.ATContentTypes.content.base import ATCTFileContent
from Products.ATContentTypes.content.base import updateAliases
from Products.ATContentTypes.content.base import registerATCT
from Products.ATContentTypes.interfaces import IATImage

from Products.ImageMag.config import *
from Products.ATColorPickerWidget.ColorPickerWidget import ColorPickerWidget
from Products.ATContentTypes.configuration import zconf

ImageMagSchema = ATFolderSchema.copy() + ATContentTypeSchema.copy() + Schema((
    IntegerField('Width',
        languageIndependent=True,
        default=300,
        required=True,
        widget=IntegerWidget(
            label='Width of result',
            label_msgid='label_width',
            description='Choose a width for the result image.',
            description_msgid='help_width',
            i18n_domain='imagemag',
        )
    ),

    IntegerField('Padding',
        languageIndependent=True,
        default=5,
        vocabulary=range(20),
        widget=SelectionWidget(
            label='Padding',
            label_msgid='label_padding',
            description='Space between images.',
            description_msgid='help_padding',
            i18n_domain='imagemag',
        )
    ),
    

    IntegerField('Margin',
        languageIndependent=True,
        default=5,
        vocabulary=range(20),
        widget=SelectionWidget(
            label='Margin',
            label_msgid='label_margin',
            description='Space around result image.',
            description_msgid='help_margin',
            i18n_domain='imagemag',
        )
    ),

    StringField('BkgColor',
        languageIndependent=True,
        default="#FFFFFF",
        widget=ColorPickerWidget(
            label='Background Color',
            label_msgid='label_background_color',
            description='Select a color for result margin and padding',
            description_msgid='help_background_color',
            i18n_domain='imagemag',
        )
    ),
    
    IntegerField('Mode',
        languageIndependent=True,
        vocabulary=range(2),
        default='0',
        widget=SelectionWidget(
            label='Assembly Mode',
            label_msgid='label_assembly_mode',
            description='Select mode of images assembly.',
            description_msgid='help_assembly_mode',
            i18n_domain='imagemag',
        )
    ),

    ImageField('image',
           languageIndependent=True,
           storage = AnnotationStorage(migrate=True),
           swallowResizeExceptions = zconf.swallowImageResizeExceptions.enable,
           pil_quality = zconf.pil_config.quality,
           pil_resize_algo = zconf.pil_config.resize_algo,
           max_size = zconf.ATImage.max_image_dimension,
           sizes= {'large'   : (768, 768),
                   'preview' : (400, 400),
                   'mini'    : (200, 200),
                   'thumb'   : (128, 128),
                   'tile'    :  (64, 64),
                   'icon'    :  (32, 32),
                   'listing' :  (16, 16),
                  },
           widget = ComputedWidget(
                    description = "",
                    label= "Image",
                    label_msgid = "label_image",
                    i18n_domain = "plone",
                    show_content_type = True,)),
    ), 
    )

#Finalize schema definition
finalizeATCTSchema(ImageMagSchema)

class ImageMag(ATFolder, ATCTFileContent):
    """Image Mags"""

    __implements__ = (ATFolder.__implements__, 
                     ATCTFileContent.__implements__,IATImage)
                     

    schema          = ImageMagSchema
    content_icon    = 'imagemag_icon.gif'
    portal_type     = 'ImageMag'
    meta_type       = 'ImageMag'
    archetype_name  = 'ImageMag'
    immediate_view = 'folder_contents'
    default_view   = 'imagemag_render_view'
    suppl_views    = ('atct_album_view',)
    typeDescription = 'Images Magazine Layout'
    typeDescMsgId   = 'description_edit_imagemag'

    # Allow images to be uploaded into this containerish object
    allowed_content_types = ['Image']
    
    # Make sure we get title-to-id generation when an object is created
    _at_rename_after_creation = True

    actions=  (
       {'action':      '''string:$object_url/imagemag_render_view''',
        'category':    '''folder''',
        'id':          '''view''',
        'name':        '''View''',
        'permissions': ('''View''',),
        'condition'  : 'python:1'},
       {'action':      '''string:$object_url/merge''',
        'category':    '''folder''',
        'id':          '''merge''',
        'name':        '''Merge''',
        'permissions': ('''Modify portal content''',),
        'condition'  : 'python:1'},
       {'action':      '''string:$object_url/atct_album_view''',
        'category':    '''folder''',
        'id':          'album',
        'name':        'Album',
        'permissions': ('''View''',),
        'condition'  : 'python:1'},
          )

    # aliases = updateAliases(ATFolder,{
       # '(Default)': 'image',
       # 'index.html': 'image',
       # })

    # def canSetDefaultPage(self):
        # return False

    security = ClassSecurityInfo()

    security.declarePrivate('Join_Two_Images')
    def Join_Two_Images(self,im1,im2,w,padding=5,margin=0,BkgColor='#FFFFFF'):
        """
        +-----------+-----------+
        |           |           |
        |  Image 1  |  Image 2  |
        |           |           |
        +-----------+-----------+
        """
        r1=float(im1.size[0])/im1.size[1]
        r2=float(im2.size[0])/im2.size[1]

        w1=int((w-(2*margin+padding))*r1/(r1+r2))
        w2=w-w1-2*margin-padding
        h1=int(w1/r1)
        h=h1+2*margin
        im1=im1.resize((w1,h1))
        im2=im2.resize((w2,h1))

        result=Image.new('RGB',(w,h),BkgColor)
        dummy=im1.copy()
        result.paste(im1,(margin,margin))
        dummy=im2.copy()
        result.paste(im2,(w1+margin+padding,margin))

        sortie=StringIO()
        result.save(sortie,'PNG')
        sortie.seek(0)
        return sortie.getvalue()



    security.declarePrivate('Join_Three_Images')
    def Join_Three_Images(self,im1,im2,im3,w=450,padding=5,margin=0,BkgColor='#FFFFFF'):
        """
        +-----------+-----------+
        |           |           |
        |  Image 1  |           |
        |           |           |
        +-----------+  Image 3  +
        |           |           |
        |  Image 2  |           |
        |           |           |
        +-----------+-----------+
        """
        r1=float(im1.size[0])/im1.size[1]
        r2=float(im2.size[0])/im2.size[1]
        r3=float(im3.size[0])/im3.size[1]

        w1=int(((w-((1+r3)*padding+2*margin)))*r1*r2/(r1*r2+r2*r3+r1*r3))
        w2=w-w1-padding-2*margin
        h1=int(w1/r1)
        h2=int(w1/r2)
        h3=int(w2/r3)
        h=h3+2*margin
        im1=im1.resize((w1,h1))
        im2=im2.resize((w1,h2))
        im3=im3.resize((w2,h3))

        result=Image.new('RGB',(w,h),BkgColor)
        dummy=im1.copy()
        result.paste(im1,(margin,margin))
        dummy=im2.copy()
        result.paste(im2,(margin,h1+padding+margin))
        dummy=im3.copy()
        result.paste(im3,(w1+padding+margin,margin))

        sortie=StringIO()
        result.save(sortie,'PNG')
        sortie.seek(0)
        return sortie.getvalue()

    security.declarePrivate('Join_Four_Images')
    def Join_Four_Images(self,im1,im2,im3,im4,w,padding=5,margin=0,BkgColor='#FFFFFF',mode=1):
        """
        mode 0
        +-----------+-----------+
        |           |           |
        |  Image 1  |           |
        |           |           |
        +-----------+           +
        |           |           |
        |  Image 2  |  Image 4  |
        |           |           |
        +-----------+           |
        |           |           |
        |  Image 3  |           |
        |           |           |
        +-----------+-----------+

        mode 1
        +-----------+-----------+
        |           |           |
        |  Image 1  |  Image 3  |
        |           |           |
        +-----------+-----------+
        |           |           |
        |  Image 2  |  Image 4  |
        |           |           |
        +-----------+-----------+
        """
        r1=float(im1.size[0])/im1.size[1]
        r2=float(im2.size[0])/im2.size[1]
        r3=float(im3.size[0])/im3.size[1]
        r4=float(im4.size[0])/im4.size[1]

        if int(mode) == 0:
            # 3 left hand, 1 right hand
            w1=int((w-2*margin-padding*(1+2*r4))*r1*r2*r3/(r2*r3*r4+r1*r3*r4+r1*r2*r4+r1*r2*r3))
            w2=w-w1-2*margin-padding
            h1=int(w1/r1)
            h2=int(w1/r2)
            h3=int(w1/r3)
            h4=int(w2/r4)
            im1=im1.resize((w1,h1))
            im2=im2.resize((w1,h2))
            im3=im3.resize((w1,h3))
            im4=im4.resize((w2,h4))
            h=2*margin+2*padding+h1+h2+h3
            result=Image.new('RGB',(w,h),BkgColor)
            dummy=im1.copy()
            result.paste(im1,(margin,margin))
            dummy=im2.copy()
            result.paste(im2,(margin,margin+h1+padding))
            dummy=im3.copy()
            result.paste(im3,(margin,margin+h1+h2+2*padding))
            dummy=im4.copy()
            result.paste(im4,(w1+margin+padding,margin))
            sortie=StringIO()
            result.save(sortie,'PNG')
            sortie.seek(0)
            return sortie.getvalue()

        if int(mode)==1:
            # 2 left hand, 2 right hand
            w1=int(r1*r2/(r3*r4*(r1+r2)+(r3+r4)*r1*r2)*(r3+r4)*(w-padding-2*margin))
            w2=w-w1-2*margin-padding
            h1=int(w1/r1)
            h2=int(w1/r2)
            h3=int(w2/r3)
            h4=int(w2/r4)
            im1=im1.resize((w1,h1))
            im2=im2.resize((w1,h2))
            im3=im3.resize((w2,h3))
            im4=im4.resize((w2,h4))
            h=2*margin+padding+h1+h2
            result=Image.new('RGB',(w,h),BkgColor)
            dummy=im1.copy()
            result.paste(im1,(margin,margin))
            dummy=im2.copy()
            result.paste(im2,(margin,margin+h1+padding))
            dummy=im3.copy()
            result.paste(im3,(margin+w1+padding,margin))
            dummy=im4.copy()
            result.paste(im4,(margin+w1+padding,margin+h3+padding))
            sortie=StringIO()
            result.save(sortie,'PNG')
            sortie.seek(0)
            return sortie.getvalue()

    security.declareProtected(ModifyPortalContent,'merge')
    def merge(self,REQUEST=None):
        """ creates the Magazine Image and store it in the image attribute
        """
        cv=self.contentValues()
        if cv:
            images=[Image.open(StringIO(i.data)) for i in cv]
        else:
            return
        nb=len(images)
        render=None
        if nb==1 or nb >4:
            return
        if nb == 2 :
            render=self.Join_Two_Images(images[0],images[1],self.Width,padding=self.Padding,margin=self.Margin,BkgColor=self.BkgColor)
        elif nb == 3 :
            render=self.Join_Three_Images(images[0],images[1],images[2],self.Width,padding=self.Padding,margin=self.Margin,BkgColor=self.BkgColor)
        elif nb == 4 :
            render=self.Join_Four_Images(images[0],images[1],images[2],images[3],self.Width,padding=self.Padding,margin=self.Margin,BkgColor=self.BkgColor,mode=self.Mode)

        if render :
            #import pdb; pdb.set_trace()
            field = self.getField('image')
            field.set(self, render)
        if REQUEST:            
            return REQUEST.RESPONSE.redirect(self.absolute_url()+'/view')
        else:
            return

    # IATImage implementation    
        
    security.declareProtected(View, 'tag')
    def tag(self, **kwargs):
        """Generate image tag using the api of the ImageField
        """
        return self.getField('image').tag(self, **kwargs)

    security.declareProtected(ModifyPortalContent, 'setImage')
    def setImage(self, value, refresh_exif=True, **kwargs):
        """Set id to uploaded id
        """
        pass
        
    def __bobo_traverse__(self, REQUEST, name):
        """Transparent access to image scales
        """
        if name.startswith('image'):
            field = self.getField('image')
            image = None
            if name == 'image':
                image = field.getScale(self)
            else:
                scalename = name[len('image_'):]
                if scalename in field.getAvailableSizes(self):
                    image = field.getScale(self, scale=scalename)
            if image is not None and not isinstance(image, basestring):
                # image might be None or '' for empty images
                return image
        return ATCTFileContent.__bobo_traverse__(self, REQUEST, name)
    
    security.declarePrivate('notifyModified')
    def notifyModified(self):
        """" from Products.CMFDefault.DublinCore """
        self.addCreator()
        self.setModificationDate()
        self.merge()

registerATCT(ImageMag, PROJECTNAME)